package com.example.shh.mylayout;

import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.view.View;
import android.view.ViewGroup;

/**
 * Created by shh on 2016-04-27.
 */
public class MyLayout extends ViewGroup{

    private int rowSize;
    private int columnSize;


    public MyLayout(Context context) {
        super(context);
    }

    public MyLayout(Context context, AttributeSet attrs) {
        super(context, attrs);

        TypedArray a=context.obtainStyledAttributes(attrs,R.styleable.MyLayout,0,0);

        try{
            this.rowSize =a.getInteger(R.styleable.MyLayout_rowSize,1);

            this.columnSize =a.getInteger(R.styleable.MyLayout_rowSize,1);

        }finally {
            a.recycle();
        }







    }


    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);


        int maxWidth = MeasureSpec.getSize(widthMeasureSpec);
        int maxHeight = MeasureSpec.getSize(heightMeasureSpec);


        double cellWidth= maxWidth/this.columnSize;
        double cellHeight= maxHeight/this.rowSize;


        final int count = getChildCount();

        for (int i = 0; i < count; i++) {
            final View child = getChildAt(i);
            final LayoutParams lp = (LayoutParams)child.getLayoutParams();


            /*자식뷰 크기지정*/
            /*고정크기로 설정하기 위해 MeasureSpec.EXACTLY 사용 */
            child.measure(MeasureSpec.makeMeasureSpec((int)(cellWidth*lp.columnSize),MeasureSpec.EXACTLY)
            ,MeasureSpec.makeMeasureSpec((int)(cellHeight*lp.rowSize),MeasureSpec.EXACTLY));

        }

        /*자신의 크기 지정*/
        setMeasuredDimension(maxWidth,maxHeight);
    }

    @Override
    protected void onLayout(boolean changed, int l, int t, int r, int b) {
        int maxWidth = getMeasuredWidth();
        int maxHeight = getMeasuredHeight();

        double cellWidth= maxWidth/this.columnSize;
        double cellHeight= maxHeight/this.rowSize;


        final int count = getChildCount();

        for (int i = 0; i < count; i++) {
            final View child = getChildAt(i);
            final LayoutParams lp = (LayoutParams)child.getLayoutParams();

            int cl = (int)(cellWidth*lp.columnPosition);
            int ct = (int)(cellHeight*lp.rowPosition);
            int cr = (int)(cl + cellWidth * lp.columnSize);
            int cb = (int)(ct + cellHeight * lp.rowSize);


            /*자식뷰 위치지정*/
            child.layout(cl,ct,cr,cb);

        }



    }

    /**
     * 자식 view 에 사용자 정의 레이아웃 파라미터를 정의하고 얻기위한 것이다.
     *
    * */



    @Override
    public LayoutParams generateLayoutParams(AttributeSet attrs) {
        return new MyLayout.LayoutParams(getContext(), attrs);
    }

    @Override
    protected LayoutParams generateDefaultLayoutParams() {
        return new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT);
    }

    @Override
    protected ViewGroup.LayoutParams generateLayoutParams(ViewGroup.LayoutParams p) {
        return new LayoutParams(p);
    }

    @Override
    protected boolean checkLayoutParams(ViewGroup.LayoutParams p) {
        return p instanceof LayoutParams;
    }



    /**
     *  자식 view 당 사용자 정의 레이아웃 정보
     *  이것으로 자식 view에 사용자가 정의한 속성값을 얻을 수 있다.
     */
    public static class LayoutParams extends MarginLayoutParams {


        public int rowSize;
        public int columnSize;
        public int rowPosition ;
        public int columnPosition ;

        public LayoutParams(Context c, AttributeSet attrs) {
            super(c, attrs);

            // Pull the layout param values from the layout XML during
            // inflation.  This is not needed if you don't care about
            // changing the layout behavior in XML.
            TypedArray a = c.obtainStyledAttributes(attrs, R.styleable.MyLayout);
            try {
                this.rowSize= a.getInt(R.styleable.MyLayout_rowSize, 0);
                this.columnSize =  a.getInt(R.styleable.MyLayout_columnSize, 0);

                rowPosition = a.getInt(R.styleable.MyLayout_rowPosition, 0);
                columnPosition = a.getInt(R.styleable.MyLayout_columnPosition, 0);
            }finally {
                a.recycle();
            }

        }

        public LayoutParams(int width, int height) {
            super(width, height);
        }

        public LayoutParams(ViewGroup.LayoutParams source) {
            super(source);
        }
    }
}
